'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Initialize a new `Coherent` test reporter.
 *
 * @param {Runner} runner
 * @api public
 */

var CoherentReporter = function (_events$EventEmitter) {
  (0, _inherits3.default)(CoherentReporter, _events$EventEmitter);

  function CoherentReporter(baseReporter, config) {
    var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
    (0, _classCallCheck3.default)(this, CoherentReporter);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(CoherentReporter).call(this));

    _this.baseReporter = baseReporter;

    _this.failSymbol = _this.baseReporter.color('fail', _this.baseReporter.symbols.err);
    _this.okSymbol = _this.baseReporter.color('checkmark', _this.baseReporter.symbols.ok);

    var epilogue = _this.baseReporter.epilogue;


    if (config && config.coherentReporterOpts) {
      _this.formatMessage = config.coherentReporterOpts.formatMessage || _this._format;
    }

    _this.outOfBandMessages = {};

    _this.on('coherent:message', function () {
      var event = arguments[0];
      var currentSuite = event.currentSuite;
      var currentTest = event.currentTest;


      var msg = this.formatMessage(event);

      if (!!currentSuite) {
        var suiteId = currentSuite.cid;
        var suiteMessages = this.outOfBandMessages[suiteId];

        if (!!currentTest) {
          suiteMessages.testsStarted = true;
          var testMessages = suiteMessages.testMessages[currentTest.title];
          if (!testMessages) {
            testMessages = [];
            suiteMessages.testMessages[currentTest.title] = testMessages;
          }
          testMessages.push(msg);
        } else {
          if (suiteMessages.testsStarted) {
            // No tests have been started...
            suiteMessages.beforeMessages.push(msg);
          } else {
            // There is no current test, and they had already started, so
            suiteMessages.afterMessages.push(msg);
          }
        }
      }
    });

    _this.on('suite:start', function (suite) {
      this.outOfBandMessages[suite.cid] = {
        beforeMessages: [],
        testMessages: {},
        testsStarted: false,
        afterMessages: []
      };
    });

    _this.on('suite:end', function (suite) {
      this._reportSuite(suite.cid);
    });
    return _this;
  }

  (0, _createClass3.default)(CoherentReporter, [{
    key: '_format',
    value: function _format(event) {
      return event.message;
    }
  }, {
    key: '_color',
    value: function _color(colorName, s) {
      return this.baseReporter.color(colorName, s);
    }
  }, {
    key: '_reportSuite',
    value: function _reportSuite(suiteId) {
      var stats = this.baseReporter.stats;

      var runner = stats.runners[suiteId];
      var capabilities = runner.capabilities;
      var specs = runner.specs;


      var browserName = this._color('bright yellow', capabilities.browserName);

      if (specs === undefined) {
        console.log('No specs for runner on ' + browserName);
        console.log();
        return;
      }

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this._hashIterate(specs)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var spec = _step.value;
          var suites = spec.suites;

          if (suites === undefined || 0 === suites.length) {
            continue;
          }

          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = (0, _getIterator3.default)(this._hashIterate(suites)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var suite = _step2.value;
              var cid = suite.cid;
              var tests = suite.tests;
              var title = suite.title;

              var suiteTitle = this._color('suite', title);

              if (tests === undefined || tests.length === 0) {
                console.log(suiteTitle + ', running on ' + browserName + ' has no tests.');
                continue;
              }

              var suiteMessages = this.outOfBandMessages[cid];
              if (suiteMessages && suiteMessages.beforeMessages) {
                for (var msg in suiteMessages.beforeMessages) {
                  console.log(msg);
                }
              }

              console.log(suiteTitle + ', running on ' + browserName + '.');
              var _iteratorNormalCompletion3 = true;
              var _didIteratorError3 = false;
              var _iteratorError3 = undefined;

              try {
                for (var _iterator3 = (0, _getIterator3.default)(this._hashIterate(tests)), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                  var test = _step3.value;

                  var testMessages = suiteMessages && suiteMessages.testMessages[test.title] || [];
                  for (var _msg2 in testMessages) {
                    console.log(_msg2);
                  }

                  switch (test.state) {
                    case 'fail':
                      console.log(this._color('fail', '  ' + this.baseReporter.symbols.err + ' ' + test.title + ' (' + stats._duration + 'ms)'));
                      console.log(this._color('error stack', '  ' + test.error.stack));
                      break;
                    case 'success':
                      console.log(this._color('pass', '  ' + this.baseReporter.symbols.okSymbol + ' ' + test.title + ' (' + stats._duration + 'ms)'));
                      break;
                    case '':
                    case 'pending':
                      // There is no state -- this test is probably pending.
                      console.log(this._color('pending', '  - ' + test.title));
                      break;
                    default:
                      console.log('Unknown state \'' + test.state + '\'; test: ' + (0, _stringify2.default)(test));
                      break;
                  }
                }
              } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion3 && _iterator3.return) {
                    _iterator3.return();
                  }
                } finally {
                  if (_didIteratorError3) {
                    throw _iteratorError3;
                  }
                }
              }

              if (suiteMessages && suiteMessages.afterMessages) {
                for (var _msg in suiteMessages.afterMessages) {
                  console.log(_msg);
                }
              }

              console.log();
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }, {
    key: '_hashIterate',
    value: _regenerator2.default.mark(function _hashIterate(hash) {
      var _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, key;

      return _regenerator2.default.wrap(function _hashIterate$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _iteratorNormalCompletion4 = true;
              _didIteratorError4 = false;
              _iteratorError4 = undefined;
              _context.prev = 3;
              _iterator4 = (0, _getIterator3.default)((0, _keys2.default)(hash));

            case 5:
              if (_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done) {
                _context.next = 12;
                break;
              }

              key = _step4.value;
              _context.next = 9;
              return hash[key];

            case 9:
              _iteratorNormalCompletion4 = true;
              _context.next = 5;
              break;

            case 12:
              _context.next = 18;
              break;

            case 14:
              _context.prev = 14;
              _context.t0 = _context['catch'](3);
              _didIteratorError4 = true;
              _iteratorError4 = _context.t0;

            case 18:
              _context.prev = 18;
              _context.prev = 19;

              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }

            case 21:
              _context.prev = 21;

              if (!_didIteratorError4) {
                _context.next = 24;
                break;
              }

              throw _iteratorError4;

            case 24:
              return _context.finish(21);

            case 25:
              return _context.finish(18);

            case 26:
            case 'end':
              return _context.stop();
          }
        }
      }, _hashIterate, this, [[3, 14, 18, 26], [19,, 21, 25]]);
    })
  }]);
  return CoherentReporter;
}(_events2.default.EventEmitter);

exports.default = CoherentReporter;
module.exports = exports['default'];